unit RWConfig;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, StdCtrls;

type
  TConfig = class(TForm)
    GroupBox1: TGroupBox;
    OKButton: TButton;
    CancelButton: TButton;
    HelpButton: TButton;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    CheckBox6: TCheckBox;
    CheckBox7: TCheckBox;
    CheckBox8: TCheckBox;
    CheckBox9: TCheckBox;
    procedure CmOK(Sender: TObject);
    procedure CmCancel(Sender: TObject);
    procedure CmHelp(Sender: TObject);
    procedure CmCheckBox(Sender: TObject);
    function UpdateMoves(moves: Integer): Integer;
  end;

var
  ConfigDlg: TConfig;
  state: Integer;
  temp_state: Integer;
  ul_list, ur_list, lr_list, ll_list: array of Integer;

// Exported Functions
function CAShowConfig(AHandle: THandle; ACaption: String): Longint; stdcall;
procedure CACloseConfig(AFormRef: Longint); stdcall;

implementation
{$R *.DFM}

// Show the config dialog
function CAShowConfig(AHandle: THandle; ACaption: String): Longint;
begin
  Application.Handle := AHandle;
  ConfigDlg := TConfig.Create(Application);
  CAShowConfig := Longint(ConfigDlg);
  ConfigDlg.Caption := 'RandomWalk';
  ConfigDlg.BorderIcons := ConfigDlg.BorderIcons - [biSystemMenu];
  ConfigDlg.BorderStyle := bsSingle;
  temp_state := ConfigDlg.UpdateMoves(state);
  ConfigDlg.ShowModal;
end;

// Close the config dialog
procedure CACloseConfig(AFormRef: Longint);
begin
  if AFormRef > 0 then
    TConfig(AFormRef).Free;
end;

// Okay accept config changes
procedure TConfig.CmOK(Sender: TObject);
var
  ul_temp, ur_temp, lr_temp, ll_temp, i: Integer;
  ul_size, ur_size, lr_size, ll_size: Integer;
begin
  state := temp_state;
  ul_temp := state and $0138;
  ur_temp := state and $01E0;
  lr_temp := state and $0183;
  ll_temp := state and $010E;
  ul_size := 0;
  ur_size := 0;
  lr_size := 0;
  ll_size := 0;

  // Count bits so we know our movement options
  for i := 1 to 9 do
  begin
    if ((ul_temp and $01) = $01) then inc(ul_size);
    if ((ur_temp and $01) = $01) then inc(ur_size);
    if ((lr_temp and $01) = $01) then inc(lr_size);
    if ((ll_temp and $01) = $01) then inc(ll_size);

    ul_temp := ul_temp shr 1;
    ur_temp := ur_temp shr 1;
    lr_temp := lr_temp shr 1;
    ll_temp := ll_temp shr 1;
  end;

  ul_temp := state and $0138;
  ur_temp := state and $01E0;
  lr_temp := state and $0183;
  ll_temp := state and $010E;
  if ul_size > 0 then SetLength(ul_list, ul_size) else ul_list := nil;
  if ur_size > 0 then SetLength(ur_list, ur_size) else ur_list := nil;
  if lr_size > 0 then SetLength(lr_list, lr_size) else lr_list := nil;
  if ll_size > 0 then SetLength(ll_list, ll_size) else ll_list := nil;

  // Alright, I don't really like this, but I'm trying to optimize
  // the CARuleMG(), better suggestions are welcome.  Basically this
  // creates an array of valid moves for each position based on
  // the bitpacked allowed moves stored in 'states'.
  i := 0;
  if ((ul_temp and $0008) = $0008) then begin ul_list[i] := 2; inc(i); end;
  if ((ul_temp and $0010) = $0010) then begin ul_list[i] := 3; inc(i); end;
  if ((ul_temp and $0020) = $0020) then begin ul_list[i] := 4; inc(i); end;
  if ((ul_temp and $0100) = $0100) then begin ul_list[i] := 1; end;

  i := 0;
  if ((ur_temp and $0020) = $0020) then begin ur_list[i] := 3; inc(i); end;
  if ((ur_temp and $0040) = $0040) then begin ur_list[i] := 4; inc(i); end;
  if ((ur_temp and $0080) = $0080) then begin ur_list[i] := 1; inc(i); end;
  if ((ur_temp and $0100) = $0100) then begin ur_list[i] := 2; end;

  i := 0;
  if ((lr_temp and $0001) = $0001) then begin lr_list[i] := 1; inc(i); end;
  if ((lr_temp and $0002) = $0002) then begin lr_list[i] := 2; inc(i); end;
  if ((lr_temp and $0080) = $0080) then begin lr_list[i] := 4; inc(i); end;
  if ((lr_temp and $0100) = $0100) then begin lr_list[i] := 3; end;

  i := 0;
  if ((ll_temp and $0002) = $0002) then begin ll_list[i] := 1; inc(i); end;
  if ((ll_temp and $0004) = $0004) then begin ll_list[i] := 2; inc(i); end;
  if ((ll_temp and $0008) = $0008) then begin ll_list[i] := 3; inc(i); end;
  if ((ll_temp and $0100) = $0100) then begin ll_list[i] := 4; end;

  ConfigDlg.Close;
end;

// Ignore new config changes
procedure TConfig.CmCancel(Sender: TObject);
begin
  ConfigDlg.Close;
end;

// A brief help spiel would be nice
procedure TConfig.CmHelp(Sender: TObject);
begin
  Application.MessageBox('Check the boxes that represent valid movements.',
                         'Help', MB_OK);
end;

// Check one of the boxes
procedure TConfig.CmCheckBox(Sender: TObject);
var
  mask, index: Integer;
begin
  index := StrToInt(TCheckBox(Sender).Caption) - 1;
  mask := $01;

  if (index >= 0) and (index <= 8) then
  begin
    mask := mask shl index;

    if (temp_state xor mask) <> 0 then
      temp_state := temp_state xor mask;

    temp_state := UpdateMoves(temp_state);
  end;
end;

// Update check boxes based on 'state'
function TConfig.UpdateMoves(moves: Integer): Integer;
var
  flag: Boolean;
  i: Integer;
begin
  UpdateMoves := moves;

  for i := 1 to 9 do
  begin
    if ((moves and $01) = $01) then flag := TRUE else flag := FALSE;
    moves := moves shr 1;

    case i of
      1:  CheckBox1.Checked := flag;
      2:  CheckBox2.Checked := flag;
      3:  CheckBox3.Checked := flag;
      4:  CheckBox4.Checked := flag;
      5:  CheckBox5.Checked := flag;
      6:  CheckBox6.Checked := flag;
      7:  CheckBox7.Checked := flag;
      8:  CheckBox8.Checked := flag;
      9:  CheckBox9.Checked := flag;
    end;
  end;
end;

end.
