// A Configureable RandomWalk DLL, Brian Behlendorf
library RandomWalk;

uses
  ShareMem,
  SysUtils,
  Windows,
  Classes,
  RWConfig in 'RWConfig.pas' {RWConfig};

// Margolus Random Walk
procedure CARuleMG(Gen, col, row: Integer;
                   var ul, ur, ll, lr: Integer); stdcall;
var
  temp, i, j: Integer;
  old: array[0..3] of Integer;
  table: array[0..3] of Integer;
begin
  old[0] := ul;         old[1] := ur;
  old[3] := ll;         old[2] := lr;
  ul := 0;              ur := 0;
  ll := 0;              lr := 0;

  for i := 0 to 3 do
    table[i] := 0;

  // Build and check a movement table,       1 | 2       -1 - Collision
  // index reflects original cell;           --+--        0 - No Cell
  // value reflects final cell.              4 | 3     1..4 - Destination
  if (Length(ul_list) > 0) then table[0] := ul_list[Random(Length(ul_list))];
  if (Length(ur_list) > 0) then table[1] := ur_list[Random(Length(ur_list))];
  if (Length(lr_list) > 0) then table[2] := lr_list[Random(Length(lr_list))];
  if (Length(ll_list) > 0) then table[3] := ll_list[Random(Length(ll_list))];

  // Flag dead cells as such.
  for i := 0 to 3 do
    if old[i] = 0 then
     table[i] := 0;

  // Find all obvious collisions that are going to happen.
  for i := 0 to 3 do
  begin
    temp := table[i];

    for j := 0 to 3 do
      if (i <> j) and (temp = table[j]) and (temp > 0) then
      begin
        table[i] := -1;
        table[j] := -1;
      end;
  end;

  // Hunt down the less obvious collisions that will result by
  // not allowing the prior collisions to take place.
  temp := 1;

  while temp = 1 do
  begin
    temp := 0;

    for i := 0 to 3 do
      if table[i] <> -1 then
        if table[table[i] - 1] = -1 then
        begin
          table[i] := -1;
          temp := 1;
        end;
  end;

  // Based on the movement table created for the cell set the
  // appropriate cells to the correst states.  (e.g. move them)
  for i := 0 to 3 do
  begin

    if (i = 0) and (table[i] = -1) then ul := old[i];
    if (i = 1) and (table[i] = -1) then ur := old[i];
    if (i = 2) and (table[i] = -1) then lr := old[i];
    if (i = 3) and (table[i] = -1) then ll := old[i];

    case table[i] of
      1: ul := old[i];          2: ur := old[i];
      4: ll := old[i];          3: lr := old[i];
    end;
  end
end;

// Setup the rule.
procedure CASetup(var RuleType, CountOfColors: Integer;
                  ColorPalette, Misc: PChar); stdcall;
begin
  RuleType := 3;
  CountOfColors := 8;
  StrCopy(ColorPalette, '8 colors');
  StrCopy(Misc, '');

  // Set default valid movements:  N, S, E, W
  state := $00AA;
  SetLength(ul_list, 2);        ul_list[0] := 2;        ul_list[1] := 4;
  SetLength(ur_list, 2);        ur_list[0] := 1;        ur_list[1] := 3;
  SetLength(lr_list, 2);        lr_list[0] := 4;        lr_list[1] := 2;
  SetLength(ll_list, 2);        ll_list[0] := 3;        ll_list[1] := 1;

  Randomize;
end;

exports
  CARuleMG      index 1,
  CASetup       index 2,
  CAShowConfig  index 3,
  CACloseConfig index 4;

procedure DLLEntryPoint(dwReason: DWord);
begin
  // Free the dynamic arrays, just to be safe.
  case dwReason of
    DLL_PROCESS_ATTACH, DLL_PROCESS_DETACH:
    begin
      ul_list := nil;           ur_list := nil;
      ul_list := nil;           lr_list := nil;
    end;
  end;
end;

begin
  DllProc := @DLLEntryPoint;
  DLLEntryPoint(DLL_PROCESS_ATTACH);
end.

